import os
import subprocess
from pathlib import Path

import click
import semver
import yaml
from click import secho
import logging

SPARK2_JAR = "org.diana-hep:histogrammar-sparksql_2.11:1.0.4"
SPARK3_JAR = "io.github.histogrammar:histogrammar_2.12:1.0.20,io.github.histogrammar:histogrammar-sparksql_2.12:1.0.20"
logger = logging.getLogger("KedroPopmon")


@click.group(name="Popmon")
def commands():
    """Kedro plugin for interactions with popmon.
    """
    pass  # pragma: no cover


@commands.group()
def popmon():
    """Run Kedro popmon commands"""


@commands.command()
def init():
    """
    Create a new popmon config file
    """
    project_path = Path.cwd()
    files = os.listdir(f"{project_path}/conf/base") + os.listdir(
        f"{project_path}/conf/local"
    )
    popmon_config_present = False
    for file in files:
        if "popmon" in file:
            popmon_config_present = True
            break
    if not popmon_config_present:
        with open(f"{project_path}/conf/base/popmon.yml", "w") as f:
            f.write("# Please enter the structure of popmon reporting\n")
            f.write(
                "# datasource: # must be one of datasources avaliable in catalog.yml, must be provided\n"
            )
            f.write(
                "# date_column: # the column you wish to refer as the data column, must be provided\n"
            )
            f.write(
                "# date_column_format: # the date format if not present equals to '%Y-%m-%d %H:%M:%S'\n"
            )
            f.write("# columns: # list of columns, by default the list is empty\n")
            f.write("# - value1 # column names you wish to analyze\n")
            f.write("# - value2\n")
        os.system("mkdir -p data/08_reporting/popmon")
    setup_pyspark_jars(project_path)
    exit(0)


popmon.add_command(init)


def get_spark_major_version():
    secho("Checking Apache Spark version, it may take a few secs...", fg="green")
    p = subprocess.Popen(
        "spark-shell --version 2> version && cat version && rm version",
        shell=True,
        stdout=subprocess.PIPE,
    )
    try:
        version = p.communicate()[0].decode("utf-8")
        start = version.index("version") + len("version")
        end = version.index("\n", start)
        version = version[start:end].strip()
        parsed = semver.parse(version)['major']
    except ValueError:
        parsed = -1
    return parsed


def setup_pyspark_jars(project_path):
    file = f"{project_path}/conf/base/spark.yml"
    conf = {}
    if os.path.exists(file):
        with open(file, "r") as yaml_file:
            conf = yaml.safe_load(yaml_file)

    with open(file, "w") as yaml_file:
        existing_jars = None
        if "spark.jars.packages" in conf.keys():
            existing_jars = set(conf.get("spark.jars.packages", "").split(","))
        else:
            existing_jars = set()
        spark_major_version = get_spark_major_version()
        if spark_major_version == 2:
            spark_jar = SPARK2_JAR
        elif spark_major_version == 3:
            spark_jar = SPARK3_JAR
        elif spark_major_version == -1:
            logger.error("Apache Spark version has not been properly detected.")
        else:
            logger.error("Detected unsupported "
                         f"Apache Spark version {spark_major_version}")
        if spark_major_version in (2, 3):
            for j in spark_jar.split(','):
                if j not in existing_jars:
                    existing_jars.add(j)
            conf.update({"spark.jars.packages": ",".join(existing_jars)})
            yaml.safe_dump(conf, yaml_file)



import logging
import os
from pathlib import Path
from typing import Any, Dict, Optional

import pandas as pd
from kedro.framework.context import load_context
from kedro.framework.hooks import hook_impl
from kedro.io import DataSetError
from popmon import df_stability_report


class KedroPopmonHook:
    def __init__(self):
        self.logger = logging.getLogger("KedroPopmon")
        self.config = {}
        self.default_date_format_pyspark = "%y-%M-%d %H:%m:%s"
        self.default_date_format = "%Y-%m-%d %H:%M:%S"

    @hook_impl
    def before_pipeline_run(self, run_params: Dict[str, Any]) -> None:
        self.context = load_context(
            project_path=run_params["project_path"],
            env=run_params["env"],
            extra_params=run_params["extra_params"],
        )
        self.config = self.context.config_loader.get("popmon*", "popmon*/**")
        self.logger.info(f"{self.config}")

    @hook_impl
    def after_pipeline_run(self, catalog: Optional[Dict[str, Dict[str, Any]]]):
        datasets = catalog._data_sets
        self.logger.info(f"Found datasets: {datasets}")
        for dataset in datasets.keys():
            if dataset in self.config:
                column = self.config[dataset]["date_column"]
                columns = self.config[dataset]["columns"]
                df_type = type(datasets[dataset]).__name__
                self.logger.info(f"Dataset type: {df_type}")

                if df_type == "CSVDataSet":
                    date_format = self.config[dataset].get(
                        "date_column_format", self.default_date_format
                    )
                    df = datasets[dataset].load()
                    self.logger.info("Using Pandas data frame API for Popmon")
                    df[column] = pd.to_datetime(df[column], format=date_format)
                elif df_type == "SparkDataSet" or df_type == "SparkHiveDataSet":
                    try:
                        import pyspark
                        from pyspark.sql.functions import to_timestamp
                        from pyspark.sql.types import DateType
                    except ImportError:
                        self.logger.error(
                            "PySpark module not found, please install it."
                        )
                    self.logger.info("Using PySpark data frame API for Popmon")
                    df = datasets[dataset].load()
                    if not isinstance(df.schema[column].dataType, DateType):
                        self.logger.info(
                            f"Adding date column with DateType "
                            f"using `{column}` column"
                        )
                        date_format = self.config[dataset].get(
                            "date_column_format", self.default_date_format_pyspark
                        )
                        df = df.withColumn(
                            column, to_timestamp(df[column], date_format,),
                        )
                    else:
                        self.logger.info(
                            "Date column in proper format, " "not adding a new one"
                        )
                else:
                    raise DataSetError("Unsupported Kedro Dataset type")
                report = df_stability_report(df, time_axis=column, features=columns)
                if "data" not in os.listdir(Path.cwd()):
                    os.mkdir("data")
                report.to_file(
                    f"data{os.path.sep}08_reporting{os.path.sep}popmon{os.path.sep}{dataset}.html"
                )


hooks = KedroPopmonHook()



import os
from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

# get the dependencies and installs
with open("requirements.txt", "r", encoding="utf-8") as f:
    requires = [x.strip() for x in f if x.strip()]

# get test dependencies and installs
with open("test_requirements.txt", "r", encoding="utf-8") as f:
    test_requires = [x.strip() for x in f if x.strip() and not x.startswith("-r")]

setup(
    name="kedro-popmon",
    version=os.getenv("VERSION", "0.1.0"),
    url="https://github.com/stephanecollot/kedro-popmon",
    author="Marian Dabrowski",
    author_email="marian.dabrowski95@gmail.com",
    description="Kedro Popmon makes integrating Popmon with Kedro easy!",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=["kedro_popmon"],
    zip_safe=False,
    include_package_data=True,
    license="MIT",
    install_requires=requires,
    tests_require=test_requires,
    extras_require={
       "spark": ["pyspark>=2.4.0", "py4j", "kedro[spark]==0.17.3"]
    },
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    entry_points={
        "kedro.project_commands": ["kedro_popmon = kedro_popmon.framework.cli.cli:commands"],
        "kedro.hooks": ["kedro_popmon = kedro_popmon.framework.kedro_popmon:hooks"],
    }
)
